/*
 * We try to find an optimal triangle grid
 */

  /* Energy has the following parts right now:
   */
  /*
   *  Edgewise expected vertex displacement
   *
   *
   *   	            Q `-_
   *              / |    `-_
   *   R' - _ _ _/_ |       `-.
   *    .       /   M - - - - - S
   *    .      /    |      _,-'
   *    .     /     |  _,-'
   *    .    /    , P '
   *    .   /  ,-'
   *    .  /,-'
   *    . /'
   *   	 R
   *
   *
   *
   *  Find R', the `expected' location of R, by
   *  reflecting S in M (the midpoint of QP).
   *
   *  Let 2d = |RR'|
   *       b = |PQ|
   *       l = |RS|
   *
   *  Giving energy contribution:
   *
   *                               2
   *                            b d
   *    E             =  F   .  ----
   *     vd, edge PQ      vd      3
   *                             l
   *
   *  (The dimensions of this are those of F_vd.)
   *
   *  By symmetry, this calculation gives the same answer with R and S
   *  exchanged.  Looking at the projection in the RMS plane:
   *
   *
   *	       	       	       S'
   *	   	    	     ,'
   *	       	       	   ,'
   *   	R'           	 ,'    	       	2d" = |SS'| = |RR'| = 2d
   *	  `-._         ,'
   *   	      `-._   ,'	      		By congruent triangles,
   *              ` M	      		with M' = midpoint of RS,
   *	       	  ,'  `-._     	       	|MM'| = |RR'|/2 = d
   *	       	,'     	  `-._
   *	      ,'       	      ` S       So use
   *	    ,'	     M' _ , - '            d = |MM'|
   *	  ,'   _ , - '
   *	 R - '
   *
   *  We choose this value for l (rather than |RM|+|MS|, say, or |RM|)
   *  because we want this symmetry and because we're happy to punish
   *  bending more than uneveness in the metric.
   *
   *  In practice to avoid division by zero we'll add epsilon to l^3
   *  and the huge energy ought then to be sufficient for the model to
   *  avoid being close to R=S.
   */

double hypotD(const double p[D3], const double q[D3]) {
  int k;
  double pq[D3];
  gsl_vector v;

  K pq[_k]= p[k] - q[k];
  v.size= D3;
  v.stride= 1;
  v.vector.data= pq;
  /* owner and block ought not to be used */

  return gsl_blas_snrm2(&v);
}

double hypotD2(const double p[D3], const double q[D3]) {
  double d2= 0;
  K d2= ffsqa(p[k] - q[k], d2);
  return d2;
}

#ifdef FP_FAST_FMA
# define ffsqa(factor,term) fma((factor),(factor),(term))
#else
# define ffsqa(factor,term) ((factor)*(factor)+(term))
#endif

static const l3_epsison= 1e-6;

static double energy_function(const double vertices[N][D3]) {
  int pi,e,qi,ri,si, k;
  double m[D3], mprime[D3], b, d2, l, sigma_bd2_l;

  FOR_EDGE(pi,e,qi) {
    ri= EDGE_END2(pi,(e+1)%V6); if (r<0) continue;
    si= EDGE_END2(pi,(e+5)%V6); if (s<0) continue;
    assert(ri == EDGE_END2(qi,(e+2)%V6));
    assert(si == EDGE_END2(qi,(e+4)%V6));
    
    K m[k]= (vertices[pi][k] + vertices[qi][k]) * 0.5;
    K mprime[k]= (vertices[ri][k] + vertices[si][k]) * 0.5;
    b= hypotD(vertices[pi], vertices[qi]);
    d2= hypotD2(m, mprime);
    l= hypotD(vertices[ri][k] - vertices[si][k]);
    l3 = l*l*l + l3_epsilon;

    sigma_bd2_l += b * d2 / l3;
  }

  

static gsl_multimin_fminimizer *minimiser;



{
  gsl_multimin_fminimizer_alloc(

